"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonDecoder = void 0;
const decodeUtf8_1 = require("../../util/buffers/utf8/decodeUtf8");
const Reader_1 = require("../../util/buffers/Reader");
const fromBase64Bin_1 = require("../../util/base64/fromBase64Bin");
const REGEX_REPLACE_ESCAPED_CHARS = /\\(b|f|n|r|t|"|\/|\\)/g;
const escapedCharReplacer = (char) => {
    switch (char) {
        case '\\b':
            return '\b';
        case '\\f':
            return '\f';
        case '\\n':
            return '\n';
        case '\\r':
            return '\r';
        case '\\t':
            return '\t';
        case '\\"':
            return '"';
        case '\\/':
            return '/';
        case '\\\\':
            return '\\';
    }
    return char;
};
const hasBinaryPrefix = (u8, x) => u8[x] === 0x64 &&
    u8[x + 1] === 0x61 &&
    u8[x + 2] === 0x74 &&
    u8[x + 3] === 0x61 &&
    u8[x + 4] === 0x3a &&
    u8[x + 5] === 0x61 &&
    u8[x + 6] === 0x70 &&
    u8[x + 7] === 0x70 &&
    u8[x + 8] === 0x6c &&
    u8[x + 9] === 0x69 &&
    u8[x + 10] === 0x63 &&
    u8[x + 11] === 0x61 &&
    u8[x + 12] === 0x74 &&
    u8[x + 13] === 0x69 &&
    u8[x + 14] === 0x6f &&
    u8[x + 15] === 0x6e &&
    u8[x + 16] === 0x2f &&
    u8[x + 17] === 0x6f &&
    u8[x + 18] === 0x63 &&
    u8[x + 19] === 0x74 &&
    u8[x + 20] === 0x65 &&
    u8[x + 21] === 0x74 &&
    u8[x + 22] === 0x2d &&
    u8[x + 23] === 0x73 &&
    u8[x + 24] === 0x74 &&
    u8[x + 25] === 0x72 &&
    u8[x + 26] === 0x65 &&
    u8[x + 27] === 0x61 &&
    u8[x + 28] === 0x6d &&
    u8[x + 29] === 0x3b &&
    u8[x + 30] === 0x62 &&
    u8[x + 31] === 0x61 &&
    u8[x + 32] === 0x73 &&
    u8[x + 33] === 0x65 &&
    u8[x + 34] === 0x36 &&
    u8[x + 35] === 0x34 &&
    u8[x + 36] === 0x2c;
const findEndingQuote = (uint8, x) => {
    const len = uint8.length;
    let char = uint8[x];
    let prev = 0;
    while (x < len) {
        if (char === 34 && prev !== 92)
            break;
        if (char === 92 && prev === 92)
            prev = 0;
        else
            prev = char;
        char = uint8[++x];
    }
    if (x === len)
        throw new Error('Invalid JSON');
    return x;
};
class JsonDecoder {
    constructor() {
        this.reader = new Reader_1.Reader();
    }
    read(uint8) {
        this.reader.reset(uint8);
        return this.readAny();
    }
    readAny() {
        this.skipWhitespace();
        const reader = this.reader;
        const x = reader.x;
        const uint8 = reader.uint8;
        const char = uint8[x];
        switch (char) {
            case 34:
                return uint8[x + 1] === 0x64
                    ? this.tryReadBin() || this.readStr()
                    : this.readStr();
            case 91:
                return this.readArr();
            case 102:
                return this.readFalse();
            case 110:
                return this.readNull();
            case 116:
                return this.readTrue();
            case 123:
                return this.readObj();
            default:
                if ((char >= 48 && char <= 57) || char === 45)
                    return this.readNum();
                throw new Error('Invalid JSON');
        }
    }
    skipWhitespace() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        let x = reader.x;
        let char = 0;
        while (true) {
            char = uint8[x];
            switch (char) {
                case 32:
                case 9:
                case 10:
                case 13:
                    x++;
                    continue;
                default:
                    reader.x = x;
                    return;
            }
        }
    }
    readNull() {
        if (this.reader.u32() !== 0x6e756c6c)
            throw new Error('Invalid JSON');
        return null;
    }
    readTrue() {
        if (this.reader.u32() !== 0x74727565)
            throw new Error('Invalid JSON');
        return true;
    }
    readFalse() {
        const reader = this.reader;
        if (reader.u8() !== 0x66 || reader.u32() !== 0x616c7365)
            throw new Error('Invalid JSON');
        return false;
    }
    readBool() {
        const reader = this.reader;
        switch (reader.uint8[reader.x]) {
            case 102:
                return this.readFalse();
            case 116:
                return this.readTrue();
            default:
                throw new Error('Invalid JSON');
        }
    }
    readNum() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        let x = reader.x;
        let str = '';
        let char = uint8[x];
        while ((char >= 45 && char <= 57) || char === 43 || char === 69 || char === 101) {
            str += String.fromCharCode(char);
            char = uint8[++x];
        }
        const num = +str;
        if (num !== num)
            throw new Error('Invalid JSON');
        reader.x = x;
        return num;
    }
    readStr() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        const char = uint8[reader.x++];
        if (char !== 0x22)
            throw new Error('Invalid JSON');
        const x0 = reader.x;
        const x1 = findEndingQuote(uint8, x0);
        let str = (0, decodeUtf8_1.decodeUtf8)(uint8, x0, x1 - x0);
        str = str.replace(REGEX_REPLACE_ESCAPED_CHARS, escapedCharReplacer);
        reader.x = x1 + 1;
        return str;
    }
    tryReadBin() {
        const reader = this.reader;
        const u8 = reader.uint8;
        let x = reader.x;
        if (u8[x++] !== 0x22)
            return undefined;
        const hasDataUrlPrefix = hasBinaryPrefix(u8, x);
        if (!hasDataUrlPrefix)
            return undefined;
        x += 37;
        const x0 = x;
        x = findEndingQuote(u8, x);
        reader.x = x0;
        const bin = (0, fromBase64Bin_1.fromBase64Bin)(reader.view, x0, x - x0);
        reader.x = x + 1;
        return bin;
    }
    readBin() {
        const reader = this.reader;
        const u8 = reader.uint8;
        let x = reader.x;
        if (u8[x++] !== 0x22)
            throw new Error('Invalid JSON');
        const hasDataUrlPrefix = hasBinaryPrefix(u8, x);
        if (!hasDataUrlPrefix)
            throw new Error('Invalid JSON');
        x += 37;
        const x0 = x;
        x = findEndingQuote(u8, x);
        reader.x = x0;
        const bin = (0, fromBase64Bin_1.fromBase64Bin)(reader.view, x0, x - x0);
        reader.x = x + 1;
        return bin;
    }
    readArr() {
        const reader = this.reader;
        if (reader.u8() !== 0x5b)
            throw new Error('Invalid JSON');
        const arr = [];
        const uint8 = reader.uint8;
        while (true) {
            this.skipWhitespace();
            const char = uint8[reader.x];
            if (char === 0x5d)
                return reader.x++, arr;
            if (char === 0x2c) {
                reader.x++;
                continue;
            }
            arr.push(this.readAny());
        }
    }
    readObj() {
        const reader = this.reader;
        if (reader.u8() !== 0x7b)
            throw new Error('Invalid JSON');
        const obj = {};
        const uint8 = reader.uint8;
        while (true) {
            this.skipWhitespace();
            const char = uint8[reader.x];
            if (char === 0x7d)
                return reader.x++, obj;
            if (char === 0x2c) {
                reader.x++;
                continue;
            }
            const key = this.readStr();
            if (key === '__proto__')
                throw new Error('Invalid JSON');
            this.skipWhitespace();
            if (reader.u8() !== 0x3a)
                throw new Error('Invalid JSON');
            this.skipWhitespace();
            obj[key] = this.readAny();
        }
    }
}
exports.JsonDecoder = JsonDecoder;
