"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CborEncoder = void 0;
var tslib_1 = require("tslib");
var isFloat32_1 = require("../../util/buffers/isFloat32");
var CborEncoderFast_1 = require("./CborEncoderFast");
var isSafeInteger = Number.isSafeInteger;
var CborEncoder = (function (_super) {
    tslib_1.__extends(CborEncoder, _super);
    function CborEncoder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CborEncoder.prototype.writeAny = function (value) {
        switch (typeof value) {
            case 'number':
                return this.writeNumber(value);
            case 'string':
                return this.writeStr(value);
            case 'boolean':
                return this.writer.u8(0xf4 + +value);
            case 'object': {
                if (!value)
                    return this.writer.u8(0xf6);
                var constructor = value.constructor;
                switch (constructor) {
                    case Array:
                        return this.writeArr(value);
                    case Uint8Array:
                        return this.writeBin(value);
                    case Map:
                        return this.writeMap(value);
                    default:
                        return this.writeObj(value);
                }
            }
            case 'undefined':
                return this.writer.u8(0xf7);
            case 'bigint':
                return this.writeBigInt(value);
        }
    };
    CborEncoder.prototype.writeNumber = function (num) {
        if (isSafeInteger(num))
            this.writeInteger(num);
        else if (typeof num === 'bigint')
            this.writeBigInt(num);
        else
            this.writeFloat(num);
    };
    CborEncoder.prototype.writeBigInt = function (int) {
        if (int >= 0)
            this.writeBigUint(int);
        else
            this.writeBigSint(int);
    };
    CborEncoder.prototype.writeBigUint = function (uint) {
        switch (true) {
            case uint <= 23:
                this.writer.u8(0 + Number(uint));
                break;
            case uint <= 0xff:
                this.writer.u16((0x18 << 8) + Number(uint));
                break;
            case uint <= 0xffff:
                this.writer.u8u16(0x19, Number(uint));
                break;
            case uint <= 0xffffffff:
                this.writer.u8u32(0x1a, Number(uint));
                break;
            default: {
                this.writer.u8u64(0x1b, uint);
                break;
            }
        }
    };
    CborEncoder.prototype.writeBigSint = function (int) {
        var uint = -BigInt(1) - int;
        switch (true) {
            case uint <= 24:
                this.writer.u8(32 + Number(uint));
                break;
            case uint <= 0xff:
                this.writer.u16((0x38 << 8) + Number(uint));
                break;
            case uint <= 0xffff:
                this.writer.u8u16(0x39, Number(uint));
                break;
            case uint <= 0xffffffff:
                this.writer.u8u32(0x3a, Number(uint));
                break;
            default:
                this.writer.u8u64(0x3b, uint);
                break;
        }
    };
    CborEncoder.prototype.writeFloat = function (float) {
        if ((0, isFloat32_1.isFloat32)(float))
            this.writer.u8f32(0xfa, float);
        else
            this.writer.u8f64(0xfb, float);
    };
    CborEncoder.prototype.writeMap = function (map) {
        var _this = this;
        this.writeMapHdr(map.size);
        map.forEach(function (value, key) {
            _this.writeAny(key);
            _this.writeAny(value);
        });
    };
    return CborEncoder;
}(CborEncoderFast_1.CborEncoderFast));
exports.CborEncoder = CborEncoder;
