"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonDecoder = void 0;
var decodeUtf8_1 = require("../../util/buffers/utf8/decodeUtf8");
var Reader_1 = require("../../util/buffers/Reader");
var fromBase64Bin_1 = require("../../util/base64/fromBase64Bin");
var REGEX_REPLACE_ESCAPED_CHARS = /\\(b|f|n|r|t|"|\/|\\)/g;
var escapedCharReplacer = function (char) {
    switch (char) {
        case '\\b':
            return '\b';
        case '\\f':
            return '\f';
        case '\\n':
            return '\n';
        case '\\r':
            return '\r';
        case '\\t':
            return '\t';
        case '\\"':
            return '"';
        case '\\/':
            return '/';
        case '\\\\':
            return '\\';
    }
    return char;
};
var hasBinaryPrefix = function (u8, x) {
    return u8[x] === 0x64 &&
        u8[x + 1] === 0x61 &&
        u8[x + 2] === 0x74 &&
        u8[x + 3] === 0x61 &&
        u8[x + 4] === 0x3a &&
        u8[x + 5] === 0x61 &&
        u8[x + 6] === 0x70 &&
        u8[x + 7] === 0x70 &&
        u8[x + 8] === 0x6c &&
        u8[x + 9] === 0x69 &&
        u8[x + 10] === 0x63 &&
        u8[x + 11] === 0x61 &&
        u8[x + 12] === 0x74 &&
        u8[x + 13] === 0x69 &&
        u8[x + 14] === 0x6f &&
        u8[x + 15] === 0x6e &&
        u8[x + 16] === 0x2f &&
        u8[x + 17] === 0x6f &&
        u8[x + 18] === 0x63 &&
        u8[x + 19] === 0x74 &&
        u8[x + 20] === 0x65 &&
        u8[x + 21] === 0x74 &&
        u8[x + 22] === 0x2d &&
        u8[x + 23] === 0x73 &&
        u8[x + 24] === 0x74 &&
        u8[x + 25] === 0x72 &&
        u8[x + 26] === 0x65 &&
        u8[x + 27] === 0x61 &&
        u8[x + 28] === 0x6d &&
        u8[x + 29] === 0x3b &&
        u8[x + 30] === 0x62 &&
        u8[x + 31] === 0x61 &&
        u8[x + 32] === 0x73 &&
        u8[x + 33] === 0x65 &&
        u8[x + 34] === 0x36 &&
        u8[x + 35] === 0x34 &&
        u8[x + 36] === 0x2c;
};
var findEndingQuote = function (uint8, x) {
    var len = uint8.length;
    var char = uint8[x];
    var prev = 0;
    while (x < len) {
        if (char === 34 && prev !== 92)
            break;
        if (char === 92 && prev === 92)
            prev = 0;
        else
            prev = char;
        char = uint8[++x];
    }
    if (x === len)
        throw new Error('Invalid JSON');
    return x;
};
var JsonDecoder = (function () {
    function JsonDecoder() {
        this.reader = new Reader_1.Reader();
    }
    JsonDecoder.prototype.read = function (uint8) {
        this.reader.reset(uint8);
        return this.readAny();
    };
    JsonDecoder.prototype.readAny = function () {
        this.skipWhitespace();
        var reader = this.reader;
        var x = reader.x;
        var uint8 = reader.uint8;
        var char = uint8[x];
        switch (char) {
            case 34:
                return uint8[x + 1] === 0x64
                    ? this.tryReadBin() || this.readStr()
                    : this.readStr();
            case 91:
                return this.readArr();
            case 102:
                return this.readFalse();
            case 110:
                return this.readNull();
            case 116:
                return this.readTrue();
            case 123:
                return this.readObj();
            default:
                if ((char >= 48 && char <= 57) || char === 45)
                    return this.readNum();
                throw new Error('Invalid JSON');
        }
    };
    JsonDecoder.prototype.skipWhitespace = function () {
        var reader = this.reader;
        var uint8 = reader.uint8;
        var x = reader.x;
        var char = 0;
        while (true) {
            char = uint8[x];
            switch (char) {
                case 32:
                case 9:
                case 10:
                case 13:
                    x++;
                    continue;
                default:
                    reader.x = x;
                    return;
            }
        }
    };
    JsonDecoder.prototype.readNull = function () {
        if (this.reader.u32() !== 0x6e756c6c)
            throw new Error('Invalid JSON');
        return null;
    };
    JsonDecoder.prototype.readTrue = function () {
        if (this.reader.u32() !== 0x74727565)
            throw new Error('Invalid JSON');
        return true;
    };
    JsonDecoder.prototype.readFalse = function () {
        var reader = this.reader;
        if (reader.u8() !== 0x66 || reader.u32() !== 0x616c7365)
            throw new Error('Invalid JSON');
        return false;
    };
    JsonDecoder.prototype.readBool = function () {
        var reader = this.reader;
        switch (reader.uint8[reader.x]) {
            case 102:
                return this.readFalse();
            case 116:
                return this.readTrue();
            default:
                throw new Error('Invalid JSON');
        }
    };
    JsonDecoder.prototype.readNum = function () {
        var reader = this.reader;
        var uint8 = reader.uint8;
        var x = reader.x;
        var str = '';
        var char = uint8[x];
        while ((char >= 45 && char <= 57) || char === 43 || char === 69 || char === 101) {
            str += String.fromCharCode(char);
            char = uint8[++x];
        }
        var num = +str;
        if (num !== num)
            throw new Error('Invalid JSON');
        reader.x = x;
        return num;
    };
    JsonDecoder.prototype.readStr = function () {
        var reader = this.reader;
        var uint8 = reader.uint8;
        var char = uint8[reader.x++];
        if (char !== 0x22)
            throw new Error('Invalid JSON');
        var x0 = reader.x;
        var x1 = findEndingQuote(uint8, x0);
        var str = (0, decodeUtf8_1.decodeUtf8)(uint8, x0, x1 - x0);
        str = str.replace(REGEX_REPLACE_ESCAPED_CHARS, escapedCharReplacer);
        reader.x = x1 + 1;
        return str;
    };
    JsonDecoder.prototype.tryReadBin = function () {
        var reader = this.reader;
        var u8 = reader.uint8;
        var x = reader.x;
        if (u8[x++] !== 0x22)
            return undefined;
        var hasDataUrlPrefix = hasBinaryPrefix(u8, x);
        if (!hasDataUrlPrefix)
            return undefined;
        x += 37;
        var x0 = x;
        x = findEndingQuote(u8, x);
        reader.x = x0;
        var bin = (0, fromBase64Bin_1.fromBase64Bin)(reader.view, x0, x - x0);
        reader.x = x + 1;
        return bin;
    };
    JsonDecoder.prototype.readBin = function () {
        var reader = this.reader;
        var u8 = reader.uint8;
        var x = reader.x;
        if (u8[x++] !== 0x22)
            throw new Error('Invalid JSON');
        var hasDataUrlPrefix = hasBinaryPrefix(u8, x);
        if (!hasDataUrlPrefix)
            throw new Error('Invalid JSON');
        x += 37;
        var x0 = x;
        x = findEndingQuote(u8, x);
        reader.x = x0;
        var bin = (0, fromBase64Bin_1.fromBase64Bin)(reader.view, x0, x - x0);
        reader.x = x + 1;
        return bin;
    };
    JsonDecoder.prototype.readArr = function () {
        var reader = this.reader;
        if (reader.u8() !== 0x5b)
            throw new Error('Invalid JSON');
        var arr = [];
        var uint8 = reader.uint8;
        while (true) {
            this.skipWhitespace();
            var char = uint8[reader.x];
            if (char === 0x5d)
                return reader.x++, arr;
            if (char === 0x2c) {
                reader.x++;
                continue;
            }
            arr.push(this.readAny());
        }
    };
    JsonDecoder.prototype.readObj = function () {
        var reader = this.reader;
        if (reader.u8() !== 0x7b)
            throw new Error('Invalid JSON');
        var obj = {};
        var uint8 = reader.uint8;
        while (true) {
            this.skipWhitespace();
            var char = uint8[reader.x];
            if (char === 0x7d)
                return reader.x++, obj;
            if (char === 0x2c) {
                reader.x++;
                continue;
            }
            var key = this.readStr();
            if (key === '__proto__')
                throw new Error('Invalid JSON');
            this.skipWhitespace();
            if (reader.u8() !== 0x3a)
                throw new Error('Invalid JSON');
            this.skipWhitespace();
            obj[key] = this.readAny();
        }
    };
    return JsonDecoder;
}());
exports.JsonDecoder = JsonDecoder;
